# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ImageModel"]

ImageModel: TypeAlias = Literal[
    "gpt-image-1",
    "gpt-image-1-mini",
    "gpt-image-2",
    "gpt-image-2-2026-04-21",
    "gpt-image-1.5",
    "chatgpt-image-latest",
    "dall-e-2",
    "dall-e-3",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["ContainerNetworkPolicyDomainSecret"]


class ContainerNetworkPolicyDomainSecret(BaseModel):
    domain: str
    """The domain associated with the secret."""

    name: str
    """The name of the secret to inject for the domain."""

    value: str
    """The secret value to inject for the domain."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RealtimeFunctionTool"]


class RealtimeFunctionTool(BaseModel):
    description: Optional[str] = None
    """
    The description of the function, including guidance on when and how to call it,
    and guidance about what to tell the user when calling (if anything).
    """

    name: Optional[str] = None
    """The name of the function."""

    parameters: Optional[object] = None
    """Parameters of the function in JSON Schema."""

    type: Optional[Literal["function"]] = None
    """The type of the tool, i.e. `function`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["ResponseFormatJSONObject"]


class ResponseFormatJSONObject(TypedDict, total=False):
    """JSON object response format.

    An older method of generating JSON responses.
    Using `json_schema` is recommended for models that support it. Note that the
    model will not generate JSON without a system or user message instructing it
    to do so.
    """

    type: Required[Literal["json_object"]]
    """The type of response format being defined. Always `json_object`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["BetaSkillReferenceParam"]


class BetaSkillReferenceParam(TypedDict, total=False):
    skill_id: Required[str]
    """The ID of the referenced skill."""

    type: Required[Literal["skill_reference"]]
    """References a skill created with the /v1/skills endpoint."""

    version: str
    """Optional skill version. Use a positive integer or 'latest'. Omit for default."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "BetaComputerActionListParam",
    "BetaComputerActionParam",
    "Click",
    "DoubleClick",
    "Drag",
    "DragPath",
    "Keypress",
    "Move",
    "Screenshot",
    "Scroll",
    "Type",
    "Wait",
]


class Click(TypedDict, total=False):
    """A click action."""

    button: Required[Literal["left", "right", "wheel", "back", "forward"]]
    """Indicates which mouse button was pressed during the click.

    One of `left`, `right`, `wheel`, `back`, or `forward`.
    """

    type: Required[Literal["click"]]
    """Specifies the event type. For a click action, this property is always `click`."""

    x: Required[int]
    """The x-coordinate where the click occurred."""

    y: Required[int]
    """The y-coordinate where the click occurred."""

    keys: Optional[SequenceNotStr[str]]
    """The keys being held while clicking."""


class DoubleClick(TypedDict, total=False):
    """A double click action."""

    keys: Required[Optional[SequenceNotStr[str]]]
    """The keys being held while double-clicking."""

    type: Required[Literal["double_click"]]
    """Specifies the event type.

    For a double click action, this property is always set to `double_click`.
    """

    x: Required[int]
    """The x-coordinate where the double click occurred."""

    y: Required[int]
    """The y-coordinate where the double click occurred."""


class DragPath(TypedDict, total=False):
    """An x/y coordinate pair, e.g. `{ x: 100, y: 200 }`."""

    x: Required[int]
    """The x-coordinate."""

    y: Required[int]
    """The y-coordinate."""


class Drag(TypedDict, total=False):
    """A drag action."""

    path: Required[Iterable[DragPath]]
    """An array of coordinates representing the path of the drag action.

    Coordinates will appear as an array of objects, eg

    ```
    [
      { x: 100, y: 200 },
      { x: 200, y: 300 }
    ]
    ```
    """

    type: Required[Literal["drag"]]
    """Specifies the event type.

    For a drag action, this property is always set to `drag`.
    """

    keys: Optional[SequenceNotStr[str]]
    """The keys being held while dragging the mouse."""


class Keypress(TypedDict, total=False):
    """A collection of keypresses the model would like to perform."""

    keys: Required[SequenceNotStr[str]]
    """The combination of keys the model is requesting to be pressed.

    This is an array of strings, each representing a key.
    """

    type: Required[Literal["keypress"]]
    """Specifies the event type.

    For a keypress action, this property is always set to `keypress`.
    """


class Move(TypedDict, total=False):
    """A mouse move action."""

    type: Required[Literal["move"]]
    """Specifies the event type.

    For a move action, this property is always set to `move`.
    """

    x: Required[int]
    """The x-coordinate to move to."""

    y: Required[int]
    """The y-coordinate to move to."""

    keys: Optional[SequenceNotStr[str]]
    """The keys being held while moving the mouse."""


class Screenshot(TypedDict, total=False):
    """A screenshot action."""

    type: Required[Literal["screenshot"]]
    """Specifies the event type.

    For a screenshot action, this property is always set to `screenshot`.
    """


class Scroll(TypedDict, total=False):
    """A scroll action."""

    scroll_x: Required[int]
    """The horizontal scroll distance."""

    scroll_y: Required[int]
    """The vertical scroll distance."""

    type: Required[Literal["scroll"]]
    """Specifies the event type.

    For a scroll action, this property is always set to `scroll`.
    """

    x: Required[int]
    """The x-coordinate where the scroll occurred."""

    y: Required[int]
    """The y-coordinate where the scroll occurred."""

    keys: Optional[SequenceNotStr[str]]
    """The keys being held while scrolling."""


class Type(TypedDict, total=False):
    """An action to type in text."""

    text: Required[str]
    """The text to type."""

    type: Required[Literal["type"]]
    """Specifies the event type.

    For a type action, this property is always set to `type`.
    """


class Wait(TypedDict, total=False):
    """A wait action."""

    type: Required[Literal["wait"]]
    """Specifies the event type.

    For a wait action, this property is always set to `wait`.
    """


BetaComputerActionParam: TypeAlias = Union[Click, DoubleClick, Drag, Keypress, Move, Screenshot, Scroll, Type, Wait]

BetaComputerActionListParam: TypeAlias = List[BetaComputerActionParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.pagination import SyncConversationCursorPage, AsyncConversationCursorPage
from openai.types.beta.chatkit import ChatKitThread, ThreadDeleteResponse
from openai.types.beta.chatkit.chatkit_thread_item_list import Data

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestThreads:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_retrieve(self, client: OpenAI) -> None:
        thread = client.beta.chatkit.threads.retrieve(
            "cthr_123",
        )
        assert_matches_type(ChatKitThread, thread, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: OpenAI) -> None:
        response = client.beta.chatkit.threads.with_raw_response.retrieve(
            "cthr_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(ChatKitThread, thread, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: OpenAI) -> None:
        with client.beta.chatkit.threads.with_streaming_response.retrieve(
            "cthr_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = response.parse()
            assert_matches_type(ChatKitThread, thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            client.beta.chatkit.threads.with_raw_response.retrieve(
                "",
            )

    @parametrize
    def test_method_list(self, client: OpenAI) -> None:
        thread = client.beta.chatkit.threads.list()
        assert_matches_type(SyncConversationCursorPage[ChatKitThread], thread, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: OpenAI) -> None:
        thread = client.beta.chatkit.threads.list(
            after="after",
            before="before",
            limit=0,
            order="asc",
            user="x",
        )
        assert_matches_type(SyncConversationCursorPage[ChatKitThread], thread, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: OpenAI) -> None:
        response = client.beta.chatkit.threads.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(SyncConversationCursorPage[ChatKitThread], thread, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: OpenAI) -> None:
        with client.beta.chatkit.threads.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = response.parse()
            assert_matches_type(SyncConversationCursorPage[ChatKitThread], thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: OpenAI) -> None:
        thread = client.beta.chatkit.threads.delete(
            "cthr_123",
        )
        assert_matches_type(ThreadDeleteResponse, thread, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: OpenAI) -> None:
        response = client.beta.chatkit.threads.with_raw_response.delete(
            "cthr_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(ThreadDeleteResponse, thread, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: OpenAI) -> None:
        with client.beta.chatkit.threads.with_streaming_response.delete(
            "cthr_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = response.parse()
            assert_matches_type(ThreadDeleteResponse, thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            client.beta.chatkit.threads.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_list_items(self, client: OpenAI) -> None:
        thread = client.beta.chatkit.threads.list_items(
            thread_id="cthr_123",
        )
        assert_matches_type(SyncConversationCursorPage[Data], thread, path=["response"])

    @parametrize
    def test_method_list_items_with_all_params(self, client: OpenAI) -> None:
        thread = client.beta.chatkit.threads.list_items(
            thread_id="cthr_123",
            after="after",
            before="before",
            limit=0,
            order="asc",
        )
        assert_matches_type(SyncConversationCursorPage[Data], thread, path=["response"])

    @parametrize
    def test_raw_response_list_items(self, client: OpenAI) -> None:
        response = client.beta.chatkit.threads.with_raw_response.list_items(
            thread_id="cthr_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(SyncConversationCursorPage[Data], thread, path=["response"])

    @parametrize
    def test_streaming_response_list_items(self, client: OpenAI) -> None:
        with client.beta.chatkit.threads.with_streaming_response.list_items(
            thread_id="cthr_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = response.parse()
            assert_matches_type(SyncConversationCursorPage[Data], thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_items(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            client.beta.chatkit.threads.with_raw_response.list_items(
                thread_id="",
            )


class TestAsyncThreads:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOpenAI) -> None:
        thread = await async_client.beta.chatkit.threads.retrieve(
            "cthr_123",
        )
        assert_matches_type(ChatKitThread, thread, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.chatkit.threads.with_raw_response.retrieve(
            "cthr_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(ChatKitThread, thread, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.chatkit.threads.with_streaming_response.retrieve(
            "cthr_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = await response.parse()
            assert_matches_type(ChatKitThread, thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            await async_client.beta.chatkit.threads.with_raw_response.retrieve(
                "",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncOpenAI) -> None:
        thread = await async_client.beta.chatkit.threads.list()
        assert_matches_type(AsyncConversationCursorPage[ChatKitThread], thread, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOpenAI) -> None:
        thread = await async_client.beta.chatkit.threads.list(
            after="after",
            before="before",
            limit=0,
            order="asc",
            user="x",
        )
        assert_matches_type(AsyncConversationCursorPage[ChatKitThread], thread, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.chatkit.threads.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(AsyncConversationCursorPage[ChatKitThread], thread, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.chatkit.threads.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = await response.parse()
            assert_matches_type(AsyncConversationCursorPage[ChatKitThread], thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncOpenAI) -> None:
        thread = await async_client.beta.chatkit.threads.delete(
            "cthr_123",
        )
        assert_matches_type(ThreadDeleteResponse, thread, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.chatkit.threads.with_raw_response.delete(
            "cthr_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(ThreadDeleteResponse, thread, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.chatkit.threads.with_streaming_response.delete(
            "cthr_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = await response.parse()
            assert_matches_type(ThreadDeleteResponse, thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            await async_client.beta.chatkit.threads.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_list_items(self, async_client: AsyncOpenAI) -> None:
        thread = await async_client.beta.chatkit.threads.list_items(
            thread_id="cthr_123",
        )
        assert_matches_type(AsyncConversationCursorPage[Data], thread, path=["response"])

    @parametrize
    async def test_method_list_items_with_all_params(self, async_client: AsyncOpenAI) -> None:
        thread = await async_client.beta.chatkit.threads.list_items(
            thread_id="cthr_123",
            after="after",
            before="before",
            limit=0,
            order="asc",
        )
        assert_matches_type(AsyncConversationCursorPage[Data], thread, path=["response"])

    @parametrize
    async def test_raw_response_list_items(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.chatkit.threads.with_raw_response.list_items(
            thread_id="cthr_123",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        thread = response.parse()
        assert_matches_type(AsyncConversationCursorPage[Data], thread, path=["response"])

    @parametrize
    async def test_streaming_response_list_items(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.chatkit.threads.with_streaming_response.list_items(
            thread_id="cthr_123",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            thread = await response.parse()
            assert_matches_type(AsyncConversationCursorPage[Data], thread, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_items(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `thread_id` but received ''"):
            await async_client.beta.chatkit.threads.with_raw_response.list_items(
                thread_id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

from .beta_inline_skill_source_param import BetaInlineSkillSourceParam

__all__ = ["BetaInlineSkillParam"]


class BetaInlineSkillParam(TypedDict, total=False):
    description: Required[str]
    """The description of the skill."""

    name: Required[str]
    """The name of the skill."""

    source: Required[BetaInlineSkillSourceParam]
    """Inline skill payload"""

    type: Required[Literal["inline"]]
    """Defines an inline skill for this request."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.types import ContentProvenanceCheck

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestContentProvenanceChecks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: OpenAI) -> None:
        content_provenance_check = client.content_provenance_checks.create(
            file=b"Example data",
        )
        assert_matches_type(ContentProvenanceCheck, content_provenance_check, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: OpenAI) -> None:
        response = client.content_provenance_checks.with_raw_response.create(
            file=b"Example data",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        content_provenance_check = response.parse()
        assert_matches_type(ContentProvenanceCheck, content_provenance_check, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: OpenAI) -> None:
        with client.content_provenance_checks.with_streaming_response.create(
            file=b"Example data",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            content_provenance_check = response.parse()
            assert_matches_type(ContentProvenanceCheck, content_provenance_check, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncContentProvenanceChecks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOpenAI) -> None:
        content_provenance_check = await async_client.content_provenance_checks.create(
            file=b"Example data",
        )
        assert_matches_type(ContentProvenanceCheck, content_provenance_check, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.content_provenance_checks.with_raw_response.create(
            file=b"Example data",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        content_provenance_check = response.parse()
        assert_matches_type(ContentProvenanceCheck, content_provenance_check, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOpenAI) -> None:
        async with async_client.content_provenance_checks.with_streaming_response.create(
            file=b"Example data",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            content_provenance_check = await response.parse()
            assert_matches_type(ContentProvenanceCheck, content_provenance_check, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ...._models import BaseModel
from .chatkit_response_output_text import ChatKitResponseOutputText

__all__ = ["ChatKitThreadAssistantMessageItem"]


class ChatKitThreadAssistantMessageItem(BaseModel):
    """Assistant-authored message within a thread."""

    id: str
    """Identifier of the thread item."""

    content: List[ChatKitResponseOutputText]
    """Ordered assistant response segments."""

    created_at: int
    """Unix timestamp (in seconds) for when the item was created."""

    object: Literal["chatkit.thread_item"]
    """Type discriminator that is always `chatkit.thread_item`."""

    thread_id: str
    """Identifier of the parent thread."""

    type: Literal["chatkit.assistant_message"]
    """Type discriminator that is always `chatkit.assistant_message`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BetaToolChoiceTypes"]


class BetaToolChoiceTypes(BaseModel):
    """
    Indicates that the model should use a built-in tool to generate a response.
    [Learn more about built-in tools](https://platform.openai.com/docs/guides/tools).
    """

    type: Literal[
        "file_search",
        "web_search_preview",
        "computer",
        "computer_use_preview",
        "computer_use",
        "web_search_preview_2025_03_11",
        "image_generation",
        "code_interpreter",
    ]
    """The type of hosted tool the model should to use.

    Learn more about
    [built-in tools](https://platform.openai.com/docs/guides/tools).

    Allowed values are:

    - `file_search`
    - `web_search_preview`
    - `computer`
    - `computer_use_preview`
    - `computer_use`
    - `code_interpreter`
    - `image_generation`
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .container_auto import ContainerAuto
from .local_environment import LocalEnvironment
from .container_reference import ContainerReference

__all__ = ["FunctionShellTool", "Environment"]

Environment: TypeAlias = Annotated[
    Union[ContainerAuto, LocalEnvironment, ContainerReference, None], PropertyInfo(discriminator="type")
]


class FunctionShellTool(BaseModel):
    """A tool that allows the model to execute shell commands."""

    type: Literal["shell"]
    """The type of the shell tool. Always `shell`."""

    allowed_callers: Optional[List[Literal["direct", "programmatic"]]] = None
    """The tool invocation context(s)."""

    environment: Optional[Environment] = None



from __future__ import annotations

from openai import OpenAI
from openai.types.beta import BetaResponseOutputItem

PROMPT = """Compare these fictional proposals on speed, cost, reliability, and security.
Proposal Alpha: 2-week launch, $40k, managed vendor, 99.9% SLA, data leaves the VPC.
Proposal Beta: 6-week launch, $70k, self-hosted, 99.5% target, data stays in the VPC.
Delegate each proposal to a separate agent, then compare their evidence and recommend one."""


def agent_name(item: BetaResponseOutputItem) -> str:
    return item.agent.agent_name if item.agent else "/root"


client = OpenAI()

with client.beta.responses.connect(
    extra_headers={"OpenAI-Beta": "responses_multi_agent=v1"},
) as connection:
    connection.send(
        {
            "type": "response.create",
            "model": "gpt-5.6-sol",
            "input": PROMPT,
            "multi_agent": {"enabled": True},
        }
    )

    item_agents: dict[int, str] = {}
    labeled_items: set[int] = set()
    for event in connection:
        if event.type == "response.output_item.added":
            item_agents[event.output_index] = agent_name(event.item)
        elif event.type == "response.output_text.delta":
            name = item_agents.get(event.output_index, "/root")
            if event.output_index not in labeled_items:
                separator = "\n\n" if labeled_items else ""
                role = "Coordinator" if name == "/root" else "Agent"
                print(f"{separator}━━━ {role}: {name} ━━━\n")
                labeled_items.add(event.output_index)
            print(event.delta, end="", flush=True)
        if event.type == "response.completed":
            break

print()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional

from ..._models import BaseModel

__all__ = ["ChatKitWorkflow", "Tracing"]


class Tracing(BaseModel):
    """Tracing settings applied to the workflow."""

    enabled: bool
    """Indicates whether tracing is enabled."""


class ChatKitWorkflow(BaseModel):
    """Workflow metadata and state returned for the session."""

    id: str
    """Identifier of the workflow backing the session."""

    state_variables: Optional[Dict[str, Union[str, bool, float]]] = None
    """State variable key-value pairs applied when invoking the workflow.

    Defaults to null when no overrides were provided.
    """

    tracing: Tracing
    """Tracing settings applied to the workflow."""

    version: Optional[str] = None
    """Specific workflow version used for the session.

    Defaults to null when using the latest deployment.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["CertificateActivateResponse", "CertificateDetails"]


class CertificateDetails(BaseModel):
    expires_at: Optional[int] = None
    """The Unix timestamp (in seconds) of when the certificate expires."""

    valid_at: Optional[int] = None
    """The Unix timestamp (in seconds) of when the certificate becomes valid."""


class CertificateActivateResponse(BaseModel):
    """Represents an individual certificate configured at the project level."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    active: bool
    """Whether the certificate is currently active at the project level."""

    certificate_details: CertificateDetails

    created_at: int
    """The Unix timestamp (in seconds) of when the certificate was uploaded."""

    name: Optional[str] = None
    """The name of the certificate."""

    object: Literal["organization.project.certificate"]
    """The object type, which is always `organization.project.certificate`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ProjectModelPermissionsDeleted"]


class ProjectModelPermissionsDeleted(BaseModel):
    """Confirmation payload returned after deleting project model permissions."""

    deleted: bool
    """Whether the project model permissions were deleted."""

    object: Literal["project.model_permissions.deleted"]
    """The object type, which is always `project.model_permissions.deleted`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["OrganizationSpendLimit", "Enforcement"]


class Enforcement(BaseModel):
    """The current enforcement state of the hard spend limit."""

    status: Union[str, Literal["inactive", "enforcing"]]
    """Whether the hard spend limit is currently enforcing."""


class OrganizationSpendLimit(BaseModel):
    """Represents a hard spend limit configured at the organization level."""

    currency: Union[str, Literal["USD"]]
    """The currency for the threshold amount. Currently, only `USD` is supported."""

    enforcement: Enforcement
    """The current enforcement state of the hard spend limit."""

    interval: Union[str, Literal["month"]]
    """The time interval for evaluating spend against the threshold.

    Currently, only `month` is supported.
    """

    object: Literal["organization.spend_limit"]
    """The object type, which is always `organization.spend_limit`."""

    threshold_amount: int
    """The hard spend limit amount, in cents."""



from __future__ import annotations

import os
import sys
import email
import zipfile
import tempfile
import subprocess
from pathlib import Path

_SMOKE_TEST = r"""
import os
import sys
import importlib.abc
from pathlib import Path

import httpx


class BlockBotocore(importlib.abc.MetaPathFinder):
    def find_spec(self, fullname, path=None, target=None):
        if fullname == "botocore" or fullname.startswith("botocore."):
            raise ImportError(f"unexpected AWS import: {fullname}")
        return None


wheel_root = Path(os.environ["OPENAI_WHEEL_ROOT"]).resolve()
blocker = BlockBotocore()
sys.meta_path.insert(0, blocker)

import openai
from openai import OpenAI
from openai.providers import bedrock

assert Path(openai.__file__).resolve().is_relative_to(wheel_root), openai.__file__

requests = []


def handler(request):
    requests.append(request)
    return httpx.Response(200, request=request, json={})


http_client = httpx.Client(transport=httpx.MockTransport(handler), trust_env=False)
with OpenAI(
    provider=bedrock(region="us-east-1", api_key="bearer-token"),
    http_client=http_client,
) as client:
    client.get("/models", cast_to=httpx.Response)

assert requests[0].headers["Authorization"] == "Bearer bearer-token"
assert not any(name == "botocore" or name.startswith("botocore.") for name in sys.modules)

sys.meta_path.remove(blocker)
requests.clear()
http_client = httpx.Client(transport=httpx.MockTransport(handler), trust_env=False)
with OpenAI(
    provider=bedrock(
        region="us-east-1",
        access_key_id="fixture-access-key",
        secret_access_key="fixture-secret-key",
        session_token="fixture-session-token",
    ),
    http_client=http_client,
) as client:
    client.get("/models", cast_to=httpx.Response)

assert "Credential=fixture-access-key/" in requests[0].headers["Authorization"]
assert requests[0].headers["X-Amz-Security-Token"] == "fixture-session-token"
"""


def main() -> None:
    wheels = list(Path("dist").glob("*.whl"))
    if len(wheels) != 1:
        raise RuntimeError(f"Expected exactly one wheel in dist/, found: {wheels}")

    wheel = wheels[0]
    with tempfile.TemporaryDirectory() as directory:
        wheel_root = Path(directory)
        with zipfile.ZipFile(wheel) as archive:
            metadata_names = [name for name in archive.namelist() if name.endswith(".dist-info/METADATA")]
            if len(metadata_names) != 1:
                raise RuntimeError(f"Expected exactly one METADATA file in {wheel}, found: {metadata_names}")

            metadata = email.message_from_bytes(archive.read(metadata_names[0]))
            archive.extractall(wheel_root)

        requirements = metadata.get_all("Requires-Dist", [])
        botocore_requirements = [requirement for requirement in requirements if requirement.startswith("botocore")]
        if len(botocore_requirements) != 1:
            raise RuntimeError(
                f"Expected one Botocore requirement for supported Python versions: {botocore_requirements}"
            )
        if any("[crt]" in requirement for requirement in botocore_requirements):
            raise RuntimeError(
                f"The Bedrock extra must not install the unused botocore CRT extra: {botocore_requirements}"
            )
        if not all("extra == 'bedrock'" in requirement for requirement in botocore_requirements):
            raise RuntimeError(f"Botocore requirements must belong to the Bedrock extra: {botocore_requirements}")
        if any("python_version" in requirement for requirement in botocore_requirements):
            raise RuntimeError(f"Botocore requirements have redundant Python markers: {botocore_requirements}")

        environment = os.environ.copy()
        for name in (
            "AWS_ACCESS_KEY_ID",
            "AWS_SECRET_ACCESS_KEY",
            "AWS_SESSION_TOKEN",
            "AWS_PROFILE",
            "AWS_REGION",
            "AWS_DEFAULT_REGION",
            "AWS_BEARER_TOKEN_BEDROCK",
            "AWS_BEDROCK_BASE_URL",
            "OPENAI_API_KEY",
            "OPENAI_ORG_ID",
            "OPENAI_PROJECT_ID",
            "OPENAI_CUSTOM_HEADERS",
        ):
            environment.pop(name, None)
        environment["OPENAI_WHEEL_ROOT"] = str(wheel_root)
        environment["PYTHONPATH"] = str(wheel_root)
        subprocess.run([sys.executable, "-c", _SMOKE_TEST], cwd=wheel_root, env=environment, check=True)


if __name__ == "__main__":
    main()



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from .... import _legacy_response
from ...._types import Body, Query, Headers, NotGiven, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ...._base_client import make_request_options
from ....types.admin.organization import data_retention_update_params
from ....types.admin.organization.organization_data_retention import OrganizationDataRetention

__all__ = ["DataRetention", "AsyncDataRetention"]


class DataRetention(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DataRetentionWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return DataRetentionWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataRetentionWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return DataRetentionWithStreamingResponse(self)

    def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationDataRetention:
        """Retrieves organization data retention controls."""
        return self._get(
            "/organization/data_retention",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=OrganizationDataRetention,
        )

    def update(
        self,
        *,
        retention_type: Literal[
            "zero_data_retention",
            "modified_abuse_monitoring",
            "enhanced_zero_data_retention",
            "enhanced_modified_abuse_monitoring",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationDataRetention:
        """
        Updates organization data retention controls.

        Args:
          retention_type: The desired organization data retention type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/organization/data_retention",
            body=maybe_transform(
                {"retention_type": retention_type}, data_retention_update_params.DataRetentionUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=OrganizationDataRetention,
        )


class AsyncDataRetention(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDataRetentionWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataRetentionWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataRetentionWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncDataRetentionWithStreamingResponse(self)

    async def retrieve(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationDataRetention:
        """Retrieves organization data retention controls."""
        return await self._get(
            "/organization/data_retention",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=OrganizationDataRetention,
        )

    async def update(
        self,
        *,
        retention_type: Literal[
            "zero_data_retention",
            "modified_abuse_monitoring",
            "enhanced_zero_data_retention",
            "enhanced_modified_abuse_monitoring",
        ],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> OrganizationDataRetention:
        """
        Updates organization data retention controls.

        Args:
          retention_type: The desired organization data retention type.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/organization/data_retention",
            body=await async_maybe_transform(
                {"retention_type": retention_type}, data_retention_update_params.DataRetentionUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=OrganizationDataRetention,
        )


class DataRetentionWithRawResponse:
    def __init__(self, data_retention: DataRetention) -> None:
        self._data_retention = data_retention

        self.retrieve = _legacy_response.to_raw_response_wrapper(
            data_retention.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            data_retention.update,
        )


class AsyncDataRetentionWithRawResponse:
    def __init__(self, data_retention: AsyncDataRetention) -> None:
        self._data_retention = data_retention

        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            data_retention.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            data_retention.update,
        )


class DataRetentionWithStreamingResponse:
    def __init__(self, data_retention: DataRetention) -> None:
        self._data_retention = data_retention

        self.retrieve = to_streamed_response_wrapper(
            data_retention.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            data_retention.update,
        )


class AsyncDataRetentionWithStreamingResponse:
    def __init__(self, data_retention: AsyncDataRetention) -> None:
        self._data_retention = data_retention

        self.retrieve = async_to_streamed_response_wrapper(
            data_retention.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            data_retention.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ......_models import BaseModel

__all__ = ["APIKeyCreateResponse"]


class APIKeyCreateResponse(BaseModel):
    id: str
    """The identifier of the API key."""

    created_at: int
    """The Unix timestamp (in seconds) when the API key was created."""

    name: str
    """The name of the API key."""

    object: Literal["organization.project.service_account.api_key"]
    """The object type, which is always `organization.project.service_account.api_key`"""

    value: str
    """The unredacted API key value."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["AdminAPIKeyListParams"]


class AdminAPIKeyListParams(TypedDict, total=False):
    after: Optional[str]
    """Return keys with IDs that come after this ID in the pagination order."""

    limit: int
    """Maximum number of keys to return."""

    order: Literal["asc", "desc"]
    """Order results by creation time, ascending or descending."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["GroupListParams"]


class GroupListParams(TypedDict, total=False):
    after: str
    """A cursor for use in pagination.

    `after` is a group ID that defines your place in the list. For instance, if you
    make a list request and receive 100 objects, ending with group_abc, your
    subsequent call can include `after=group_abc` in order to fetch the next page of
    the list.
    """

    limit: int
    """A limit on the number of groups to be returned.

    Limit can range between 0 and 1000, and the default is 100.
    """

    order: Literal["asc", "desc"]
    """Specifies the sort order of the returned groups."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .beta_response_format_text_json_schema_config_param import BetaResponseFormatTextJSONSchemaConfigParam

__all__ = ["BetaResponseFormatTextConfigParam", "Text", "JSONObject"]


class Text(TypedDict, total=False):
    """Default response format. Used to generate text responses."""

    type: Required[Literal["text"]]
    """The type of response format being defined. Always `text`."""


class JSONObject(TypedDict, total=False):
    """JSON object response format.

    An older method of generating JSON responses.
    Using `json_schema` is recommended for models that support it. Note that the
    model will not generate JSON without a system or user message instructing it
    to do so.
    """

    type: Required[Literal["json_object"]]
    """The type of response format being defined. Always `json_object`."""


BetaResponseFormatTextConfigParam: TypeAlias = Union[Text, BetaResponseFormatTextJSONSchemaConfigParam, JSONObject]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal

import httpx

from .... import _legacy_response
from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ....pagination import SyncConversationCursorPage, AsyncConversationCursorPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.admin.organization import audit_log_list_params
from ....types.admin.organization.audit_log_list_response import AuditLogListResponse

__all__ = ["AuditLogs", "AsyncAuditLogs"]


class AuditLogs(SyncAPIResource):
    """List user actions and configuration changes within this organization."""

    @cached_property
    def with_raw_response(self) -> AuditLogsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AuditLogsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AuditLogsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AuditLogsWithStreamingResponse(self)

    def list(
        self,
        *,
        actor_emails: SequenceNotStr[str] | Omit = omit,
        actor_ids: SequenceNotStr[str] | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        effective_at: audit_log_list_params.EffectiveAt | Omit = omit,
        event_types: List[
            Literal[
                "api_key.created",
                "api_key.updated",
                "api_key.deleted",
                "certificate.created",
                "certificate.updated",
                "certificate.deleted",
                "certificates.activated",
                "certificates.deactivated",
                "checkpoint.permission.created",
                "checkpoint.permission.deleted",
                "external_key.registered",
                "external_key.removed",
                "group.created",
                "group.updated",
                "group.deleted",
                "invite.sent",
                "invite.accepted",
                "invite.deleted",
                "ip_allowlist.created",
                "ip_allowlist.updated",
                "ip_allowlist.deleted",
                "ip_allowlist.config.activated",
                "ip_allowlist.config.deactivated",
                "login.succeeded",
                "login.failed",
                "logout.succeeded",
                "logout.failed",
                "organization.updated",
                "project.created",
                "project.updated",
                "project.archived",
                "project.deleted",
                "rate_limit.updated",
                "rate_limit.deleted",
                "resource.deleted",
                "tunnel.created",
                "tunnel.updated",
                "tunnel.deleted",
                "workload_identity_provider.created",
                "workload_identity_provider.updated",
                "workload_identity_provider.deleted",
                "workload_identity_provider_mapping.created",
                "workload_identity_provider_mapping.updated",
                "workload_identity_provider_mapping.deleted",
                "role.created",
                "role.updated",
                "role.deleted",
                "role.assignment.created",
                "role.assignment.deleted",
                "role.bound_to_resource",
                "role.unbound_from_resource",
                "scim.enabled",
                "scim.disabled",
                "service_account.created",
                "service_account.updated",
                "service_account.deleted",
                "user.added",
                "user.updated",
                "user.deleted",
                "tenant.metadata.updated",
                "tenant.microsoft_entra_mapping.upserted",
                "tenant.microsoft_entra_mapping.deleted",
                "tenant.workload_identity.provider.created",
                "tenant.workload_identity.provider.updated",
                "tenant.workload_identity.provider.archived",
                "tenant.workload_identity.mapping.created",
                "tenant.workload_identity.mapping.updated",
                "tenant.workload_identity.mapping.archived",
                "tenant.workload_identity.binding.created",
                "tenant.workload_identity.principal.provisioned",
                "tenant.admin_api_key.created",
                "tenant.admin_api_key.updated",
                "tenant.admin_api_key.deleted",
                "tenant.project_api_key.created",
                "tenant.chatgpt_access_token.revoked",
                "tenant.migration.completed",
                "tenant.sso.migrated",
                "tenant.domains.migrated",
                "tenant.sso_connection.created",
                "tenant.sso_connection.updated",
                "tenant.sso_connection.deleted",
                "tenant.sso_connection.setup.started",
                "tenant.policy.created",
                "tenant.policy.updated",
                "tenant.policy.deleted",
                "tenant.policy.attached",
                "tenant.policy.detached",
                "tenant.principal_authentication_policy.resolved",
                "tenant.scim.setup.started",
                "tenant.scim.deletion.requested",
                "tenant.scim.directory.created",
                "tenant.product_access_policy.updated",
                "tenant.resource_share_grant.created",
                "tenant.resource_share_grant.updated",
                "tenant.resource_share_grant.accepted",
                "tenant.resource_share_grant.declined",
                "tenant.resource_share_grant.revoked",
                "tenant.resource_share_grant.deleted",
                "tenant.service_account.updated",
                "tenant.service_account.deleted",
                "tenant.service_account.token.revoked",
                "tenant.billing.overage_limit.updated",
                "tenant.billing.alerts.updated",
                "tenant.billing.info.updated",
                "tenant.usage_limit.workspace.updated",
                "tenant.usage_limit.group.updated",
                "tenant.usage_limit.user.updated",
                "tenant.usage_limit.increase_request.updated",
                "tenant.usage_limit.increase_request.resolved",
                "tenant.group.created",
                "tenant.group.updated",
                "tenant.group.deleted",
                "tenant.group.member.added",
                "tenant.group.member.removed",
                "tenant.migration_rollout.status.updated",
                "tenant.migration_rollout.tier.updated",
                "tenant.role.metadata.updated",
                "tenant.custom_role.created",
                "tenant.custom_role.updated",
                "tenant.custom_role.deleted",
                "tenant.role_assignment.created",
                "tenant.role_assignment.deleted",
                "tenant.resource_role_assignment.created",
                "tenant.resource_role_assignment.deleted",
                "tenant.resource_access.updated",
                "tenant.resource_access.deleted",
                "tenant.session_policy.created",
                "tenant.session_policy.updated",
                "tenant.session_policy.deleted",
                "tenant.session_revocation.started",
                "tenant.third_party_app_policy.updated",
                "tenant.user.added",
                "tenant.user.updated",
                "tenant.user.removed",
                "tenant.user.looked_up",
                "tenant.user.invited",
                "tenant.membership.revoked",
                "tenant.api_organization_invite.upserted",
                "tenant.api_organization_invite.deleted",
                "tenant.chatgpt_workspace_invite.upserted",
                "tenant.membership.accepted",
                "tenant.membership.declined",
                "tenant.workspace_invite_email_settings.updated",
            ]
        ]
        | Omit = omit,
        limit: int | Omit = omit,
        project_ids: SequenceNotStr[str] | Omit = omit,
        resource_ids: SequenceNotStr[str] | Omit = omit,
        tenant_only: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncConversationCursorPage[AuditLogListResponse]:
        """
        List user actions and configuration changes within this organization.

        Args:
          actor_emails: Return only events performed by users with these emails.

          actor_ids: Return only events performed by these actors. Can be a user ID, a service
              account ID, or an api key tracking ID.

          after: A cursor for use in pagination. `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          before: A cursor for use in pagination. `before` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              starting with obj_foo, your subsequent call can include before=obj_foo in order
              to fetch the previous page of the list.

          effective_at: Return only events whose `effective_at` (Unix seconds) is in this range.

          event_types: Return only events with a `type` in one of these values. For example,
              `project.created`. For all options, see the documentation for the
              [audit log object](https://platform.openai.com/docs/api-reference/audit-logs/object).

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          project_ids: Return only events for these projects.

          resource_ids: Return only events performed on these targets. For example, a project ID
              updated. For ChatGPT connector role events, use the workspace connector resource
              ID shown in `details.id`, such as `<workspace_id>__<connector_id>`.

          tenant_only: Return only tenant-scoped events associated with this organization. Required for
              tenant-scoped events such as `role.bound_to_resource` and
              `role.unbound_from_resource`. When `true`, all supplied event types must be
              tenant-scoped.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organization/audit_logs",
            page=SyncConversationCursorPage[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "actor_emails": actor_emails,
                        "actor_ids": actor_ids,
                        "after": after,
                        "before": before,
                        "effective_at": effective_at,
                        "event_types": event_types,
                        "limit": limit,
                        "project_ids": project_ids,
                        "resource_ids": resource_ids,
                        "tenant_only": tenant_only,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=AuditLogListResponse,
        )


class AsyncAuditLogs(AsyncAPIResource):
    """List user actions and configuration changes within this organization."""

    @cached_property
    def with_raw_response(self) -> AsyncAuditLogsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAuditLogsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAuditLogsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncAuditLogsWithStreamingResponse(self)

    def list(
        self,
        *,
        actor_emails: SequenceNotStr[str] | Omit = omit,
        actor_ids: SequenceNotStr[str] | Omit = omit,
        after: str | Omit = omit,
        before: str | Omit = omit,
        effective_at: audit_log_list_params.EffectiveAt | Omit = omit,
        event_types: List[
            Literal[
                "api_key.created",
                "api_key.updated",
                "api_key.deleted",
                "certificate.created",
                "certificate.updated",
                "certificate.deleted",
                "certificates.activated",
                "certificates.deactivated",
                "checkpoint.permission.created",
                "checkpoint.permission.deleted",
                "external_key.registered",
                "external_key.removed",
                "group.created",
                "group.updated",
                "group.deleted",
                "invite.sent",
                "invite.accepted",
                "invite.deleted",
                "ip_allowlist.created",
                "ip_allowlist.updated",
                "ip_allowlist.deleted",
                "ip_allowlist.config.activated",
                "ip_allowlist.config.deactivated",
                "login.succeeded",
                "login.failed",
                "logout.succeeded",
                "logout.failed",
                "organization.updated",
                "project.created",
                "project.updated",
                "project.archived",
                "project.deleted",
                "rate_limit.updated",
                "rate_limit.deleted",
                "resource.deleted",
                "tunnel.created",
                "tunnel.updated",
                "tunnel.deleted",
                "workload_identity_provider.created",
                "workload_identity_provider.updated",
                "workload_identity_provider.deleted",
                "workload_identity_provider_mapping.created",
                "workload_identity_provider_mapping.updated",
                "workload_identity_provider_mapping.deleted",
                "role.created",
                "role.updated",
                "role.deleted",
                "role.assignment.created",
                "role.assignment.deleted",
                "role.bound_to_resource",
                "role.unbound_from_resource",
                "scim.enabled",
                "scim.disabled",
                "service_account.created",
                "service_account.updated",
                "service_account.deleted",
                "user.added",
                "user.updated",
                "user.deleted",
                "tenant.metadata.updated",
                "tenant.microsoft_entra_mapping.upserted",
                "tenant.microsoft_entra_mapping.deleted",
                "tenant.workload_identity.provider.created",
                "tenant.workload_identity.provider.updated",
                "tenant.workload_identity.provider.archived",
                "tenant.workload_identity.mapping.created",
                "tenant.workload_identity.mapping.updated",
                "tenant.workload_identity.mapping.archived",
                "tenant.workload_identity.binding.created",
                "tenant.workload_identity.principal.provisioned",
                "tenant.admin_api_key.created",
                "tenant.admin_api_key.updated",
                "tenant.admin_api_key.deleted",
                "tenant.project_api_key.created",
                "tenant.chatgpt_access_token.revoked",
                "tenant.migration.completed",
                "tenant.sso.migrated",
                "tenant.domains.migrated",
                "tenant.sso_connection.created",
                "tenant.sso_connection.updated",
                "tenant.sso_connection.deleted",
                "tenant.sso_connection.setup.started",
                "tenant.policy.created",
                "tenant.policy.updated",
                "tenant.policy.deleted",
                "tenant.policy.attached",
                "tenant.policy.detached",
                "tenant.principal_authentication_policy.resolved",
                "tenant.scim.setup.started",
                "tenant.scim.deletion.requested",
                "tenant.scim.directory.created",
                "tenant.product_access_policy.updated",
                "tenant.resource_share_grant.created",
                "tenant.resource_share_grant.updated",
                "tenant.resource_share_grant.accepted",
                "tenant.resource_share_grant.declined",
                "tenant.resource_share_grant.revoked",
                "tenant.resource_share_grant.deleted",
                "tenant.service_account.updated",
                "tenant.service_account.deleted",
                "tenant.service_account.token.revoked",
                "tenant.billing.overage_limit.updated",
                "tenant.billing.alerts.updated",
                "tenant.billing.info.updated",
                "tenant.usage_limit.workspace.updated",
                "tenant.usage_limit.group.updated",
                "tenant.usage_limit.user.updated",
                "tenant.usage_limit.increase_request.updated",
                "tenant.usage_limit.increase_request.resolved",
                "tenant.group.created",
                "tenant.group.updated",
                "tenant.group.deleted",
                "tenant.group.member.added",
                "tenant.group.member.removed",
                "tenant.migration_rollout.status.updated",
                "tenant.migration_rollout.tier.updated",
                "tenant.role.metadata.updated",
                "tenant.custom_role.created",
                "tenant.custom_role.updated",
                "tenant.custom_role.deleted",
                "tenant.role_assignment.created",
                "tenant.role_assignment.deleted",
                "tenant.resource_role_assignment.created",
                "tenant.resource_role_assignment.deleted",
                "tenant.resource_access.updated",
                "tenant.resource_access.deleted",
                "tenant.session_policy.created",
                "tenant.session_policy.updated",
                "tenant.session_policy.deleted",
                "tenant.session_revocation.started",
                "tenant.third_party_app_policy.updated",
                "tenant.user.added",
                "tenant.user.updated",
                "tenant.user.removed",
                "tenant.user.looked_up",
                "tenant.user.invited",
                "tenant.membership.revoked",
                "tenant.api_organization_invite.upserted",
                "tenant.api_organization_invite.deleted",
                "tenant.chatgpt_workspace_invite.upserted",
                "tenant.membership.accepted",
                "tenant.membership.declined",
                "tenant.workspace_invite_email_settings.updated",
            ]
        ]
        | Omit = omit,
        limit: int | Omit = omit,
        project_ids: SequenceNotStr[str] | Omit = omit,
        resource_ids: SequenceNotStr[str] | Omit = omit,
        tenant_only: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AuditLogListResponse, AsyncConversationCursorPage[AuditLogListResponse]]:
        """
        List user actions and configuration changes within this organization.

        Args:
          actor_emails: Return only events performed by users with these emails.

          actor_ids: Return only events performed by these actors. Can be a user ID, a service
              account ID, or an api key tracking ID.

          after: A cursor for use in pagination. `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          before: A cursor for use in pagination. `before` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              starting with obj_foo, your subsequent call can include before=obj_foo in order
              to fetch the previous page of the list.

          effective_at: Return only events whose `effective_at` (Unix seconds) is in this range.

          event_types: Return only events with a `type` in one of these values. For example,
              `project.created`. For all options, see the documentation for the
              [audit log object](https://platform.openai.com/docs/api-reference/audit-logs/object).

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          project_ids: Return only events for these projects.

          resource_ids: Return only events performed on these targets. For example, a project ID
              updated. For ChatGPT connector role events, use the workspace connector resource
              ID shown in `details.id`, such as `<workspace_id>__<connector_id>`.

          tenant_only: Return only tenant-scoped events associated with this organization. Required for
              tenant-scoped events such as `role.bound_to_resource` and
              `role.unbound_from_resource`. When `true`, all supplied event types must be
              tenant-scoped.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/organization/audit_logs",
            page=AsyncConversationCursorPage[AuditLogListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "actor_emails": actor_emails,
                        "actor_ids": actor_ids,
                        "after": after,
                        "before": before,
                        "effective_at": effective_at,
                        "event_types": event_types,
                        "limit": limit,
                        "project_ids": project_ids,
                        "resource_ids": resource_ids,
                        "tenant_only": tenant_only,
                    },
                    audit_log_list_params.AuditLogListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=AuditLogListResponse,
        )


class AuditLogsWithRawResponse:
    def __init__(self, audit_logs: AuditLogs) -> None:
        self._audit_logs = audit_logs

        self.list = _legacy_response.to_raw_response_wrapper(
            audit_logs.list,
        )


class AsyncAuditLogsWithRawResponse:
    def __init__(self, audit_logs: AsyncAuditLogs) -> None:
        self._audit_logs = audit_logs

        self.list = _legacy_response.async_to_raw_response_wrapper(
            audit_logs.list,
        )


class AuditLogsWithStreamingResponse:
    def __init__(self, audit_logs: AuditLogs) -> None:
        self._audit_logs = audit_logs

        self.list = to_streamed_response_wrapper(
            audit_logs.list,
        )


class AsyncAuditLogsWithStreamingResponse:
    def __init__(self, audit_logs: AsyncAuditLogs) -> None:
        self._audit_logs = audit_logs

        self.list = async_to_streamed_response_wrapper(
            audit_logs.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ....._models import BaseModel

__all__ = ["ProjectHostedToolPermissions", "CodeInterpreter", "FileSearch", "ImageGeneration", "Mcp", "WebSearch"]


class CodeInterpreter(BaseModel):
    """Permission state for a single hosted tool on a project."""

    enabled: bool
    """Whether the hosted tool is enabled for the project."""


class FileSearch(BaseModel):
    """Permission state for a single hosted tool on a project."""

    enabled: bool
    """Whether the hosted tool is enabled for the project."""


class ImageGeneration(BaseModel):
    """Permission state for a single hosted tool on a project."""

    enabled: bool
    """Whether the hosted tool is enabled for the project."""


class Mcp(BaseModel):
    """Permission state for a single hosted tool on a project."""

    enabled: bool
    """Whether the hosted tool is enabled for the project."""


class WebSearch(BaseModel):
    """Permission state for a single hosted tool on a project."""

    enabled: bool
    """Whether the hosted tool is enabled for the project."""


class ProjectHostedToolPermissions(BaseModel):
    """Represents hosted tool permissions for a project."""

    code_interpreter: CodeInterpreter
    """Permission state for a single hosted tool on a project."""

    file_search: FileSearch
    """Permission state for a single hosted tool on a project."""

    image_generation: ImageGeneration
    """Permission state for a single hosted tool on a project."""

    mcp: Mcp
    """Permission state for a single hosted tool on a project."""

    web_search: WebSearch
    """Permission state for a single hosted tool on a project."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.pagination import SyncCursorPage, AsyncCursorPage
from openai.types.beta import (
    Assistant,
    AssistantDeleted,
)

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAssistants:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.create(
                model="gpt-4o",
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.create(
                model="gpt-4o",
                description="description",
                instructions="instructions",
                metadata={"foo": "string"},
                name="name",
                reasoning_effort="none",
                response_format="auto",
                temperature=1,
                tool_resources={
                    "code_interpreter": {"file_ids": ["string"]},
                    "file_search": {
                        "vector_store_ids": ["string"],
                        "vector_stores": [
                            {
                                "chunking_strategy": {"type": "auto"},
                                "file_ids": ["string"],
                                "metadata": {"foo": "string"},
                            }
                        ],
                    },
                },
                tools=[{"type": "code_interpreter"}],
                top_p=1,
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.assistants.with_raw_response.create(
                model="gpt-4o",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.assistants.with_streaming_response.create(
                model="gpt-4o",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = response.parse()
                assert_matches_type(Assistant, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.retrieve(
                "assistant_id",
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.assistants.with_raw_response.retrieve(
                "assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.assistants.with_streaming_response.retrieve(
                "assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = response.parse()
                assert_matches_type(Assistant, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
                client.beta.assistants.with_raw_response.retrieve(
                    "",
                )

    @parametrize
    def test_method_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.update(
                assistant_id="assistant_id",
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.update(
                assistant_id="assistant_id",
                description="description",
                instructions="instructions",
                metadata={"foo": "string"},
                model="gpt-5",
                name="name",
                reasoning_effort="none",
                response_format="auto",
                temperature=1,
                tool_resources={
                    "code_interpreter": {"file_ids": ["string"]},
                    "file_search": {"vector_store_ids": ["string"]},
                },
                tools=[{"type": "code_interpreter"}],
                top_p=1,
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.assistants.with_raw_response.update(
                assistant_id="assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.assistants.with_streaming_response.update(
                assistant_id="assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = response.parse()
                assert_matches_type(Assistant, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
                client.beta.assistants.with_raw_response.update(
                    assistant_id="",
                )

    @parametrize
    def test_method_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.list()

        assert_matches_type(SyncCursorPage[Assistant], assistant, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.list(
                after="after",
                before="before",
                limit=0,
                order="asc",
            )

        assert_matches_type(SyncCursorPage[Assistant], assistant, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.assistants.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(SyncCursorPage[Assistant], assistant, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.assistants.with_streaming_response.list() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = response.parse()
                assert_matches_type(SyncCursorPage[Assistant], assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = client.beta.assistants.delete(
                "assistant_id",
            )

        assert_matches_type(AssistantDeleted, assistant, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.beta.assistants.with_raw_response.delete(
                "assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(AssistantDeleted, assistant, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with client.beta.assistants.with_streaming_response.delete(
                "assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = response.parse()
                assert_matches_type(AssistantDeleted, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: OpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
                client.beta.assistants.with_raw_response.delete(
                    "",
                )


class TestAsyncAssistants:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.create(
                model="gpt-4o",
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.create(
                model="gpt-4o",
                description="description",
                instructions="instructions",
                metadata={"foo": "string"},
                name="name",
                reasoning_effort="none",
                response_format="auto",
                temperature=1,
                tool_resources={
                    "code_interpreter": {"file_ids": ["string"]},
                    "file_search": {
                        "vector_store_ids": ["string"],
                        "vector_stores": [
                            {
                                "chunking_strategy": {"type": "auto"},
                                "file_ids": ["string"],
                                "metadata": {"foo": "string"},
                            }
                        ],
                    },
                },
                tools=[{"type": "code_interpreter"}],
                top_p=1,
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.assistants.with_raw_response.create(
                model="gpt-4o",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.assistants.with_streaming_response.create(
                model="gpt-4o",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = await response.parse()
                assert_matches_type(Assistant, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.retrieve(
                "assistant_id",
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.assistants.with_raw_response.retrieve(
                "assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.assistants.with_streaming_response.retrieve(
                "assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = await response.parse()
                assert_matches_type(Assistant, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
                await async_client.beta.assistants.with_raw_response.retrieve(
                    "",
                )

    @parametrize
    async def test_method_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.update(
                assistant_id="assistant_id",
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.update(
                assistant_id="assistant_id",
                description="description",
                instructions="instructions",
                metadata={"foo": "string"},
                model="gpt-5",
                name="name",
                reasoning_effort="none",
                response_format="auto",
                temperature=1,
                tool_resources={
                    "code_interpreter": {"file_ids": ["string"]},
                    "file_search": {"vector_store_ids": ["string"]},
                },
                tools=[{"type": "code_interpreter"}],
                top_p=1,
            )

        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.assistants.with_raw_response.update(
                assistant_id="assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(Assistant, assistant, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.assistants.with_streaming_response.update(
                assistant_id="assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = await response.parse()
                assert_matches_type(Assistant, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
                await async_client.beta.assistants.with_raw_response.update(
                    assistant_id="",
                )

    @parametrize
    async def test_method_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.list()

        assert_matches_type(AsyncCursorPage[Assistant], assistant, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.list(
                after="after",
                before="before",
                limit=0,
                order="asc",
            )

        assert_matches_type(AsyncCursorPage[Assistant], assistant, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.assistants.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(AsyncCursorPage[Assistant], assistant, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.assistants.with_streaming_response.list() as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = await response.parse()
                assert_matches_type(AsyncCursorPage[Assistant], assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            assistant = await async_client.beta.assistants.delete(
                "assistant_id",
            )

        assert_matches_type(AssistantDeleted, assistant, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.beta.assistants.with_raw_response.delete(
                "assistant_id",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        assistant = response.parse()
        assert_matches_type(AssistantDeleted, assistant, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.beta.assistants.with_streaming_response.delete(
                "assistant_id",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                assistant = await response.parse()
                assert_matches_type(AssistantDeleted, assistant, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `assistant_id` but received ''"):
                await async_client.beta.assistants.with_raw_response.delete(
                    "",
                )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.pagination import SyncConversationCursorPage, AsyncConversationCursorPage
from openai.types.admin.organization.projects import (
    ProjectRateLimit,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRateLimits:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_list_rate_limits(self, client: OpenAI) -> None:
        rate_limit = client.admin.organization.projects.rate_limits.list_rate_limits(
            project_id="project_id",
        )
        assert_matches_type(SyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

    @parametrize
    def test_method_list_rate_limits_with_all_params(self, client: OpenAI) -> None:
        rate_limit = client.admin.organization.projects.rate_limits.list_rate_limits(
            project_id="project_id",
            after="after",
            before="before",
            limit=0,
        )
        assert_matches_type(SyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

    @parametrize
    def test_raw_response_list_rate_limits(self, client: OpenAI) -> None:
        response = client.admin.organization.projects.rate_limits.with_raw_response.list_rate_limits(
            project_id="project_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_limit = response.parse()
        assert_matches_type(SyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

    @parametrize
    def test_streaming_response_list_rate_limits(self, client: OpenAI) -> None:
        with client.admin.organization.projects.rate_limits.with_streaming_response.list_rate_limits(
            project_id="project_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_limit = response.parse()
            assert_matches_type(SyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_list_rate_limits(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `project_id` but received ''"):
            client.admin.organization.projects.rate_limits.with_raw_response.list_rate_limits(
                project_id="",
            )

    @parametrize
    def test_method_update_rate_limit(self, client: OpenAI) -> None:
        rate_limit = client.admin.organization.projects.rate_limits.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
        )
        assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

    @parametrize
    def test_method_update_rate_limit_with_all_params(self, client: OpenAI) -> None:
        rate_limit = client.admin.organization.projects.rate_limits.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
            batch_1_day_max_input_tokens=0,
            max_audio_megabytes_per_1_minute=0,
            max_images_per_1_minute=0,
            max_requests_per_1_day=0,
            max_requests_per_1_minute=0,
            max_tokens_per_1_minute=0,
        )
        assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

    @parametrize
    def test_raw_response_update_rate_limit(self, client: OpenAI) -> None:
        response = client.admin.organization.projects.rate_limits.with_raw_response.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_limit = response.parse()
        assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

    @parametrize
    def test_streaming_response_update_rate_limit(self, client: OpenAI) -> None:
        with client.admin.organization.projects.rate_limits.with_streaming_response.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_limit = response.parse()
            assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_rate_limit(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `project_id` but received ''"):
            client.admin.organization.projects.rate_limits.with_raw_response.update_rate_limit(
                rate_limit_id="rate_limit_id",
                project_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `rate_limit_id` but received ''"):
            client.admin.organization.projects.rate_limits.with_raw_response.update_rate_limit(
                rate_limit_id="",
                project_id="project_id",
            )


class TestAsyncRateLimits:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_list_rate_limits(self, async_client: AsyncOpenAI) -> None:
        rate_limit = await async_client.admin.organization.projects.rate_limits.list_rate_limits(
            project_id="project_id",
        )
        assert_matches_type(AsyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

    @parametrize
    async def test_method_list_rate_limits_with_all_params(self, async_client: AsyncOpenAI) -> None:
        rate_limit = await async_client.admin.organization.projects.rate_limits.list_rate_limits(
            project_id="project_id",
            after="after",
            before="before",
            limit=0,
        )
        assert_matches_type(AsyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

    @parametrize
    async def test_raw_response_list_rate_limits(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.projects.rate_limits.with_raw_response.list_rate_limits(
            project_id="project_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_limit = response.parse()
        assert_matches_type(AsyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

    @parametrize
    async def test_streaming_response_list_rate_limits(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.projects.rate_limits.with_streaming_response.list_rate_limits(
            project_id="project_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_limit = await response.parse()
            assert_matches_type(AsyncConversationCursorPage[ProjectRateLimit], rate_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_list_rate_limits(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `project_id` but received ''"):
            await async_client.admin.organization.projects.rate_limits.with_raw_response.list_rate_limits(
                project_id="",
            )

    @parametrize
    async def test_method_update_rate_limit(self, async_client: AsyncOpenAI) -> None:
        rate_limit = await async_client.admin.organization.projects.rate_limits.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
        )
        assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

    @parametrize
    async def test_method_update_rate_limit_with_all_params(self, async_client: AsyncOpenAI) -> None:
        rate_limit = await async_client.admin.organization.projects.rate_limits.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
            batch_1_day_max_input_tokens=0,
            max_audio_megabytes_per_1_minute=0,
            max_images_per_1_minute=0,
            max_requests_per_1_day=0,
            max_requests_per_1_minute=0,
            max_tokens_per_1_minute=0,
        )
        assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

    @parametrize
    async def test_raw_response_update_rate_limit(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.projects.rate_limits.with_raw_response.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        rate_limit = response.parse()
        assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

    @parametrize
    async def test_streaming_response_update_rate_limit(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.projects.rate_limits.with_streaming_response.update_rate_limit(
            rate_limit_id="rate_limit_id",
            project_id="project_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            rate_limit = await response.parse()
            assert_matches_type(ProjectRateLimit, rate_limit, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_rate_limit(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `project_id` but received ''"):
            await async_client.admin.organization.projects.rate_limits.with_raw_response.update_rate_limit(
                rate_limit_id="rate_limit_id",
                project_id="",
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `rate_limit_id` but received ''"):
            await async_client.admin.organization.projects.rate_limits.with_raw_response.update_rate_limit(
                rate_limit_id="",
                project_id="project_id",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["ProjectModelPermissions"]


class ProjectModelPermissions(BaseModel):
    """Represents the model allowlist or denylist policy for a project."""

    mode: Literal["allow_list", "deny_list"]
    """Whether the project uses an allowlist or a denylist."""

    api_model_ids: List[str] = FieldInfo(alias="model_ids")
    """The model IDs included in the model permissions policy."""

    object: Literal["project.model_permissions"]
    """The object type, which is always `project.model_permissions`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing
import threading

from ._exceptions import WebSocketQueueFullError


class SendQueue:
    """Bounded byte-size queue for outgoing WebSocket messages.

    Messages are stored as pre-serialized strings. The queue enforces a
    maximum byte budget so that unbounded buffering cannot occur during
    reconnection windows.
    """

    def __init__(self, max_bytes: int = 1_048_576) -> None:
        self._queue: list[tuple[str, int]] = []  # (data, byte_length)
        self._bytes: int = 0
        self._max_bytes = max_bytes
        self._lock = threading.Lock()

    def enqueue(self, data: str) -> None:
        """Append *data* to the queue.

        Raises :class:`WebSocketQueueFullError` if the message would
        exceed the byte-size limit.
        """
        byte_length = len(data.encode("utf-8"))
        with self._lock:
            if self._bytes + byte_length > self._max_bytes:
                raise WebSocketQueueFullError("send queue is full, message discarded")
            self._queue.append((data, byte_length))
            self._bytes += byte_length

    def flush_sync(self, send: typing.Callable[[str], object]) -> None:
        """Send every queued message via *send*.

        If *send* raises, the failing message and all subsequent messages
        are re-queued and the error is re-raised.
        """
        with self._lock:
            pending = list(self._queue)
            self._queue.clear()
            self._bytes = 0

        for i, (data, _byte_length) in enumerate(pending):
            try:
                send(data)
            except Exception:
                with self._lock:
                    remaining = pending[i:]
                    self._queue = remaining + self._queue
                    self._bytes = sum(bl for _, bl in self._queue)
                raise

    async def flush_async(self, send: typing.Callable[[str], typing.Awaitable[object]]) -> None:
        """Async variant of :meth:`flush_sync`."""
        with self._lock:
            pending = list(self._queue)
            self._queue.clear()
            self._bytes = 0

        for i, (data, _byte_length) in enumerate(pending):
            try:
                await send(data)
            except Exception:
                with self._lock:
                    remaining = pending[i:]
                    self._queue = remaining + self._queue
                    self._bytes = sum(bl for _, bl in self._queue)
                raise

    def drain(self) -> list[str]:
        """Remove and return all queued messages."""
        with self._lock:
            items = [data for data, _ in self._queue]
            self._queue.clear()
            self._bytes = 0
            return items

    def __len__(self) -> int:
        with self._lock:
            return len(self._queue)

    def __bool__(self) -> bool:
        with self._lock:
            return len(self._queue) > 0



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ProjectSpendLimitDeleted"]


class ProjectSpendLimitDeleted(BaseModel):
    """Confirmation payload returned after deleting a project hard spend limit."""

    deleted: bool
    """Whether the hard spend limit was deleted."""

    object: Literal["project.spend_limit.deleted"]
    """The object type, which is always `project.spend_limit.deleted`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.pagination import SyncPage, AsyncPage, SyncConversationCursorPage, AsyncConversationCursorPage
from openai.types.admin.organization import (
    Certificate,
    CertificateListResponse,
    CertificateDeleteResponse,
    CertificateActivateResponse,
    CertificateDeactivateResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestCertificates:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.create(
            certificate="certificate",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.create(
            certificate="certificate",
            name="name",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: OpenAI) -> None:
        response = client.admin.organization.certificates.with_raw_response.create(
            certificate="certificate",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: OpenAI) -> None:
        with client.admin.organization.certificates.with_streaming_response.create(
            certificate="certificate",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = response.parse()
            assert_matches_type(Certificate, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.retrieve(
            certificate_id="certificate_id",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.retrieve(
            certificate_id="certificate_id",
            include=["content"],
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: OpenAI) -> None:
        response = client.admin.organization.certificates.with_raw_response.retrieve(
            certificate_id="certificate_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: OpenAI) -> None:
        with client.admin.organization.certificates.with_streaming_response.retrieve(
            certificate_id="certificate_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = response.parse()
            assert_matches_type(Certificate, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `certificate_id` but received ''"):
            client.admin.organization.certificates.with_raw_response.retrieve(
                certificate_id="",
            )

    @parametrize
    def test_method_update(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.update(
            certificate_id="certificate_id",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_method_update_with_all_params(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.update(
            certificate_id="certificate_id",
            name="name",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_raw_response_update(self, client: OpenAI) -> None:
        response = client.admin.organization.certificates.with_raw_response.update(
            certificate_id="certificate_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    def test_streaming_response_update(self, client: OpenAI) -> None:
        with client.admin.organization.certificates.with_streaming_response.update(
            certificate_id="certificate_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = response.parse()
            assert_matches_type(Certificate, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `certificate_id` but received ''"):
            client.admin.organization.certificates.with_raw_response.update(
                certificate_id="",
            )

    @parametrize
    def test_method_list(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.list()
        assert_matches_type(SyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.list(
            after="after",
            limit=0,
            order="asc",
        )
        assert_matches_type(SyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: OpenAI) -> None:
        response = client.admin.organization.certificates.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(SyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: OpenAI) -> None:
        with client.admin.organization.certificates.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = response.parse()
            assert_matches_type(SyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.delete(
            "certificate_id",
        )
        assert_matches_type(CertificateDeleteResponse, certificate, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: OpenAI) -> None:
        response = client.admin.organization.certificates.with_raw_response.delete(
            "certificate_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(CertificateDeleteResponse, certificate, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: OpenAI) -> None:
        with client.admin.organization.certificates.with_streaming_response.delete(
            "certificate_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = response.parse()
            assert_matches_type(CertificateDeleteResponse, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `certificate_id` but received ''"):
            client.admin.organization.certificates.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_activate(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.activate(
            certificate_ids=["cert_abc"],
        )
        assert_matches_type(SyncPage[CertificateActivateResponse], certificate, path=["response"])

    @parametrize
    def test_raw_response_activate(self, client: OpenAI) -> None:
        response = client.admin.organization.certificates.with_raw_response.activate(
            certificate_ids=["cert_abc"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(SyncPage[CertificateActivateResponse], certificate, path=["response"])

    @parametrize
    def test_streaming_response_activate(self, client: OpenAI) -> None:
        with client.admin.organization.certificates.with_streaming_response.activate(
            certificate_ids=["cert_abc"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = response.parse()
            assert_matches_type(SyncPage[CertificateActivateResponse], certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_deactivate(self, client: OpenAI) -> None:
        certificate = client.admin.organization.certificates.deactivate(
            certificate_ids=["cert_abc"],
        )
        assert_matches_type(SyncPage[CertificateDeactivateResponse], certificate, path=["response"])

    @parametrize
    def test_raw_response_deactivate(self, client: OpenAI) -> None:
        response = client.admin.organization.certificates.with_raw_response.deactivate(
            certificate_ids=["cert_abc"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(SyncPage[CertificateDeactivateResponse], certificate, path=["response"])

    @parametrize
    def test_streaming_response_deactivate(self, client: OpenAI) -> None:
        with client.admin.organization.certificates.with_streaming_response.deactivate(
            certificate_ids=["cert_abc"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = response.parse()
            assert_matches_type(SyncPage[CertificateDeactivateResponse], certificate, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncCertificates:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.create(
            certificate="certificate",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.create(
            certificate="certificate",
            name="name",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.certificates.with_raw_response.create(
            certificate="certificate",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.certificates.with_streaming_response.create(
            certificate="certificate",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = await response.parse()
            assert_matches_type(Certificate, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.retrieve(
            certificate_id="certificate_id",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.retrieve(
            certificate_id="certificate_id",
            include=["content"],
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.certificates.with_raw_response.retrieve(
            certificate_id="certificate_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.certificates.with_streaming_response.retrieve(
            certificate_id="certificate_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = await response.parse()
            assert_matches_type(Certificate, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `certificate_id` but received ''"):
            await async_client.admin.organization.certificates.with_raw_response.retrieve(
                certificate_id="",
            )

    @parametrize
    async def test_method_update(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.update(
            certificate_id="certificate_id",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.update(
            certificate_id="certificate_id",
            name="name",
        )
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_raw_response_update(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.certificates.with_raw_response.update(
            certificate_id="certificate_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(Certificate, certificate, path=["response"])

    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.certificates.with_streaming_response.update(
            certificate_id="certificate_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = await response.parse()
            assert_matches_type(Certificate, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `certificate_id` but received ''"):
            await async_client.admin.organization.certificates.with_raw_response.update(
                certificate_id="",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.list()
        assert_matches_type(AsyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.list(
            after="after",
            limit=0,
            order="asc",
        )
        assert_matches_type(AsyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.certificates.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(AsyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.certificates.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = await response.parse()
            assert_matches_type(AsyncConversationCursorPage[CertificateListResponse], certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.delete(
            "certificate_id",
        )
        assert_matches_type(CertificateDeleteResponse, certificate, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.certificates.with_raw_response.delete(
            "certificate_id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(CertificateDeleteResponse, certificate, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.certificates.with_streaming_response.delete(
            "certificate_id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = await response.parse()
            assert_matches_type(CertificateDeleteResponse, certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `certificate_id` but received ''"):
            await async_client.admin.organization.certificates.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_activate(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.activate(
            certificate_ids=["cert_abc"],
        )
        assert_matches_type(AsyncPage[CertificateActivateResponse], certificate, path=["response"])

    @parametrize
    async def test_raw_response_activate(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.certificates.with_raw_response.activate(
            certificate_ids=["cert_abc"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(AsyncPage[CertificateActivateResponse], certificate, path=["response"])

    @parametrize
    async def test_streaming_response_activate(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.certificates.with_streaming_response.activate(
            certificate_ids=["cert_abc"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = await response.parse()
            assert_matches_type(AsyncPage[CertificateActivateResponse], certificate, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_deactivate(self, async_client: AsyncOpenAI) -> None:
        certificate = await async_client.admin.organization.certificates.deactivate(
            certificate_ids=["cert_abc"],
        )
        assert_matches_type(AsyncPage[CertificateDeactivateResponse], certificate, path=["response"])

    @parametrize
    async def test_raw_response_deactivate(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.certificates.with_raw_response.deactivate(
            certificate_ids=["cert_abc"],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        certificate = response.parse()
        assert_matches_type(AsyncPage[CertificateDeactivateResponse], certificate, path=["response"])

    @parametrize
    async def test_streaming_response_deactivate(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.certificates.with_streaming_response.deactivate(
            certificate_ids=["cert_abc"],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            certificate = await response.parse()
            assert_matches_type(AsyncPage[CertificateDeactivateResponse], certificate, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .realtime_session_create_response import RealtimeSessionCreateResponse
from .realtime_transcription_session_create_response import RealtimeTranscriptionSessionCreateResponse

__all__ = ["ClientSecretCreateResponse", "Session"]

Session: TypeAlias = Annotated[
    Union[RealtimeSessionCreateResponse, RealtimeTranscriptionSessionCreateResponse], PropertyInfo(discriminator="type")
]


class ClientSecretCreateResponse(BaseModel):
    """Response from creating a session and client secret for the Realtime API."""

    expires_at: int
    """Expiration timestamp for the client secret, in seconds since epoch."""

    session: Session
    """The session configuration for either a realtime or transcription session."""

    value: str
    """The generated client secret value."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..... import _legacy_response
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from .....pagination import SyncConversationCursorPage, AsyncConversationCursorPage
from ....._base_client import AsyncPaginator, make_request_options
from .....types.admin.organization.projects import api_key_list_params
from .....types.admin.organization.projects.project_api_key import ProjectAPIKey
from .....types.admin.organization.projects.api_key_delete_response import APIKeyDeleteResponse

__all__ = ["APIKeys", "AsyncAPIKeys"]


class APIKeys(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> APIKeysWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return APIKeysWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> APIKeysWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return APIKeysWithStreamingResponse(self)

    def retrieve(
        self,
        api_key_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectAPIKey:
        """
        Retrieves an API key in the project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not api_key_id:
            raise ValueError(f"Expected a non-empty value for `api_key_id` but received {api_key_id!r}")
        return self._get(
            path_template(
                "/organization/projects/{project_id}/api_keys/{api_key_id}",
                project_id=project_id,
                api_key_id=api_key_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectAPIKey,
        )

    def list(
        self,
        project_id: str,
        *,
        after: str | Omit = omit,
        limit: int | Omit = omit,
        owner_project_access: Literal["active", "inactive", "any"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncConversationCursorPage[ProjectAPIKey]:
        """
        Returns a list of API keys in the project.

        Args:
          after: A cursor for use in pagination. `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          owner_project_access: Filter API keys by whether the owner currently has effective access to the
              project. Use `active` for owners with access, `inactive` for owners without
              access, or `any` for all enabled project API keys. If omitted, the endpoint
              applies its existing membership-based visibility rules, which may exclude some
              enabled keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._get_api_list(
            path_template("/organization/projects/{project_id}/api_keys", project_id=project_id),
            page=SyncConversationCursorPage[ProjectAPIKey],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "owner_project_access": owner_project_access,
                    },
                    api_key_list_params.APIKeyListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=ProjectAPIKey,
        )

    def delete(
        self,
        api_key_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> APIKeyDeleteResponse:
        """
        Deletes an API key from the project.

        Returns confirmation of the key deletion, or an error if the key belonged to a
        service account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not api_key_id:
            raise ValueError(f"Expected a non-empty value for `api_key_id` but received {api_key_id!r}")
        return self._delete(
            path_template(
                "/organization/projects/{project_id}/api_keys/{api_key_id}",
                project_id=project_id,
                api_key_id=api_key_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=APIKeyDeleteResponse,
        )


class AsyncAPIKeys(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAPIKeysWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAPIKeysWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAPIKeysWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncAPIKeysWithStreamingResponse(self)

    async def retrieve(
        self,
        api_key_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectAPIKey:
        """
        Retrieves an API key in the project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not api_key_id:
            raise ValueError(f"Expected a non-empty value for `api_key_id` but received {api_key_id!r}")
        return await self._get(
            path_template(
                "/organization/projects/{project_id}/api_keys/{api_key_id}",
                project_id=project_id,
                api_key_id=api_key_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectAPIKey,
        )

    def list(
        self,
        project_id: str,
        *,
        after: str | Omit = omit,
        limit: int | Omit = omit,
        owner_project_access: Literal["active", "inactive", "any"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ProjectAPIKey, AsyncConversationCursorPage[ProjectAPIKey]]:
        """
        Returns a list of API keys in the project.

        Args:
          after: A cursor for use in pagination. `after` is an object ID that defines your place
              in the list. For instance, if you make a list request and receive 100 objects,
              ending with obj_foo, your subsequent call can include after=obj_foo in order to
              fetch the next page of the list.

          limit: A limit on the number of objects to be returned. Limit can range between 1 and
              100, and the default is 20.

          owner_project_access: Filter API keys by whether the owner currently has effective access to the
              project. Use `active` for owners with access, `inactive` for owners without
              access, or `any` for all enabled project API keys. If omitted, the endpoint
              applies its existing membership-based visibility rules, which may exclude some
              enabled keys.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._get_api_list(
            path_template("/organization/projects/{project_id}/api_keys", project_id=project_id),
            page=AsyncConversationCursorPage[ProjectAPIKey],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "limit": limit,
                        "owner_project_access": owner_project_access,
                    },
                    api_key_list_params.APIKeyListParams,
                ),
                security={"admin_api_key_auth": True},
            ),
            model=ProjectAPIKey,
        )

    async def delete(
        self,
        api_key_id: str,
        *,
        project_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> APIKeyDeleteResponse:
        """
        Deletes an API key from the project.

        Returns confirmation of the key deletion, or an error if the key belonged to a
        service account.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        if not api_key_id:
            raise ValueError(f"Expected a non-empty value for `api_key_id` but received {api_key_id!r}")
        return await self._delete(
            path_template(
                "/organization/projects/{project_id}/api_keys/{api_key_id}",
                project_id=project_id,
                api_key_id=api_key_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=APIKeyDeleteResponse,
        )


class APIKeysWithRawResponse:
    def __init__(self, api_keys: APIKeys) -> None:
        self._api_keys = api_keys

        self.retrieve = _legacy_response.to_raw_response_wrapper(
            api_keys.retrieve,
        )
        self.list = _legacy_response.to_raw_response_wrapper(
            api_keys.list,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            api_keys.delete,
        )


class AsyncAPIKeysWithRawResponse:
    def __init__(self, api_keys: AsyncAPIKeys) -> None:
        self._api_keys = api_keys

        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            api_keys.retrieve,
        )
        self.list = _legacy_response.async_to_raw_response_wrapper(
            api_keys.list,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            api_keys.delete,
        )


class APIKeysWithStreamingResponse:
    def __init__(self, api_keys: APIKeys) -> None:
        self._api_keys = api_keys

        self.retrieve = to_streamed_response_wrapper(
            api_keys.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            api_keys.list,
        )
        self.delete = to_streamed_response_wrapper(
            api_keys.delete,
        )


class AsyncAPIKeysWithStreamingResponse:
    def __init__(self, api_keys: AsyncAPIKeys) -> None:
        self._api_keys = api_keys

        self.retrieve = async_to_streamed_response_wrapper(
            api_keys.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            api_keys.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            api_keys.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["GroupDeleteResponse"]


class GroupDeleteResponse(BaseModel):
    """Confirmation payload returned after deleting a group."""

    id: str
    """Identifier of the deleted group."""

    deleted: bool
    """Whether the group was deleted."""

    object: Literal["group.deleted"]
    """Always `group.deleted`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import TypeAlias

from .beta_response_input_file_content_param import BetaResponseInputFileContentParam
from .beta_response_input_text_content_param import BetaResponseInputTextContentParam
from .beta_response_input_image_content_param import BetaResponseInputImageContentParam

__all__ = ["BetaResponseFunctionCallOutputItemParam"]

BetaResponseFunctionCallOutputItemParam: TypeAlias = Union[
    BetaResponseInputTextContentParam, BetaResponseInputImageContentParam, BetaResponseInputFileContentParam
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel
from .beta_response_output_text import BetaResponseOutputText
from .beta_response_output_refusal import BetaResponseOutputRefusal

__all__ = ["BetaResponseContentPartAddedEvent", "Part", "PartReasoningText", "Agent"]


class PartReasoningText(BaseModel):
    """Reasoning text from the model."""

    text: str
    """The reasoning text from the model."""

    type: Literal["reasoning_text"]
    """The type of the reasoning text. Always `reasoning_text`."""


Part: TypeAlias = Annotated[
    Union[BetaResponseOutputText, BetaResponseOutputRefusal, PartReasoningText], PropertyInfo(discriminator="type")
]


class Agent(BaseModel):
    """The agent that owns this multi-agent streaming event."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseContentPartAddedEvent(BaseModel):
    """Emitted when a new content part is added."""

    content_index: int
    """The index of the content part that was added."""

    item_id: str
    """The ID of the output item that the content part was added to."""

    output_index: int
    """The index of the output item that the content part was added to."""

    part: Part
    """The content part that was added."""

    sequence_number: int
    """The sequence number of this event."""

    type: Literal["response.content_part.added"]
    """The type of the event. Always `response.content_part.added`."""

    agent: Optional[Agent] = None
    """The agent that owns this multi-agent streaming event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["UserUpdateParams"]


class UserUpdateParams(TypedDict, total=False):
    project_id: Required[str]

    role: Optional[str]
    """`owner` or `member`"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["TranscriptionLanguage"]


class TranscriptionLanguage(BaseModel):
    """A language detected in transcribed audio."""

    code: str
    """The code of a language detected in the audio."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["VideoSize"]

VideoSize: TypeAlias = Literal["720x1280", "1280x720", "1024x1792", "1792x1024"]
